//
//  SubChannelManager.swift
//  SampleApp
//
//  Created by Amity on 7/11/2565 BE.
//  Copyright © 2565 BE David Zhang. All rights reserved.
//

import Foundation
import AmitySDK

extension AmitySubChannel {
    
    struct ViewModel {
        let id: String
        let displayName: String
        let channelId: String
        let isDeleted: Bool
        let creatorId: String
        let unreadCount: Int
        let hasMentioned: Bool
        let latestMessageId: String?
        let object: AmitySubChannel
        let preview: AmityMessagePreview?
    }
    
    var swiftUIModel: ViewModel {
        return ViewModel(id: self.subChannelId, displayName: self.displayName, channelId: self.channelId, isDeleted: self.isDeleted, creatorId: self.creatorId, unreadCount: self.unreadCount, hasMentioned: self.isMentioned, latestMessageId: self.latestMessageId, object: self, preview: self.messagePreview)
    }
}

class SubChannelManager: ObservableObject {
    
    // Newly created subchannel list
    @Published var subChannels = [AmitySubChannel.ViewModel]()
    
    var queryToken: AmityNotificationToken?
    var singleQueryToken: AmityNotificationToken?
    var subChannelCollection: AmityCollection<AmitySubChannel>?
    
    let subChannelRepo = AmitySubChannelRepository(client: AmityManager.shared.client!)
    private var subChannel: AmityObject<AmitySubChannel>?
    
    func fetchSubChannels(channelId: String, isDeleted: Bool, excludeMainChannel: Bool, completion: @escaping ([AmitySubChannel.ViewModel], Error?) -> Void) {
        var options = AmitySubChannelQueryOptions(channelId: channelId)
        options.isDeleted = isDeleted
        options.excludeDefaultSubChannel = excludeMainChannel
        
        subChannelCollection = subChannelRepo.getSubChannels(options: options)
        queryToken  = subChannelCollection?.observe { [weak self ] liveCollection, _, error in
            
            guard let weakSelf = self else { return }
            
            if let err = error {
                completion([], err)
            } else {
                let allSubChannels = liveCollection.allObjects()
                weakSelf.subChannels = allSubChannels.map { $0.swiftUIModel }
                completion(weakSelf.subChannels, nil)
            }
        }
    }
    
    func fetchMoreSubChannels() {
        let hasMore = subChannelCollection?.hasNext ?? false
        if hasMore {
            subChannelCollection?.nextPage()
        }
    }
    
    // Single subchannel query
    func fetchSubChannel(id: String, completion: @escaping (AmitySubChannel?, Error?) -> Void) {
        subChannel = subChannelRepo.getSubChannel(withId: id)
        singleQueryToken = subChannel?.observe { [weak self] liveObject, error in
            if let err = error {
                completion(liveObject.snapshot, err)
            } else {
                if let subChannel = liveObject.snapshot {
                    self?.subChannels.removeAll()
                    self?.subChannels = [subChannel.swiftUIModel]
                    
                    Log.add(info: "SubCHANNEL COUNT: \(self?.subChannels.count ?? 0)")
                }
                completion(liveObject.snapshot, error)
            }
            
            // Invalidate Token
            self?.singleQueryToken?.invalidate()
        }
    }
    
    func createSubChannel(channelId: String, displayName: String, completion: @escaping (AmitySubChannel?, Error?) -> Void) {
        Task { @MainActor in
            do {
              let subchannel = try await subChannelRepo.createSubChannel(withName: displayName, inChannel: channelId)
                self.subChannels = [subchannel.swiftUIModel]
                completion(subchannel, nil)
            } catch {
                completion(nil, error)
            }
        }
    }
    
    func deleteSubChannel(withId id: String, hardDelete: Bool = false, completion: @escaping (Bool, Error?) -> Void) {
        Task { @MainActor in
            do {
                if hardDelete {
                    let success = try await subChannelRepo.hardDeleteSubChannel(withId: id)
                    completion(success, nil)
                } else {
                    let success = try await subChannelRepo.softDeleteSubChannel(withId: id)
                    completion(success, nil)
                }
                self.subChannels = []
            } catch {
                completion(false, error)
            }
        }
    }
    
    func updateSubChannel(displayName: String, subChannelId: String, completion: @escaping (AmitySubChannel?, Error?) -> Void) {
        Task { @MainActor in
            do {
                let subchannel = try await subChannelRepo.editSubChannel(withId: subChannelId, updatedName: displayName)
                completion(subchannel, nil)
                self.subChannels = [subchannel.swiftUIModel]
            } catch {
                completion(nil, error)
            }
        }
    }
}
