//
//  CommunityRealTimeEventController.swift
//  SampleApp
//
//  Created by Nishan Niraula on 11/24/21.
//  Copyright © 2021 David Zhang. All rights reserved.
//

import Foundation
import UIKit
import AmitySDK

class CommunityRealTimeEventController: UIViewController {
    
    let tableView = UITableView()
    
    var manager: CommunityRealTimeEventManager!
    
    override func loadView() {
        view = UIView()
        view.backgroundColor = .white
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.translatesAutoresizingMaskIntoConstraints = false
        view.addSubview(tableView)
        
        NSLayoutConstraint.activate([
            tableView.leadingAnchor.constraint(equalTo: view.leadingAnchor),
            tableView.trailingAnchor.constraint(equalTo: view.trailingAnchor),
            tableView.topAnchor.constraint(equalTo: view.topAnchor),
            tableView.bottomAnchor.constraint(equalTo: view.bottomAnchor)
        ])
        
        let nib = UINib(nibName: RealTimeEventHeaderCell.identifier, bundle: nil)
        tableView.register(nib, forCellReuseIdentifier: RealTimeEventHeaderCell.identifier)
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: UITableViewCell.identifier)
        
        tableView.dataSource = self
        tableView.delegate = self
        tableView.estimatedRowHeight = 100
        tableView.rowHeight = UITableView.automaticDimension
        tableView.separatorStyle = .none
        tableView.reloadData()
        tableView.separatorStyle = .singleLine
        tableView.separatorInset = UIEdgeInsets(top: 0, left: 24, bottom: 0, right: 24)
        
        // Get list of posts for the feed.
        fetchCommunityFeed(feedType: .published, feedSortOption: .lastCreated)
        
        // Observe community changes too
        manager.observeCommunityChanges = { [weak self] in
            self?.tableView.reloadSections([0], with: .automatic)
        }
        
        let filterButton = UIBarButtonItem(image: UIImage(systemName: "slider.horizontal.3"), style: .plain, target: self, action: #selector(onFilterButtonTap))
        self.navigationItem.rightBarButtonItem = filterButton
    }
    
    func fetchCommunityFeed(feedType: AmityFeedType, feedSortOption: AmityPostQuerySortOption) {
        manager.feedType = feedType
        manager.feedSortOption = feedSortOption
        
        self.title = manager.feedDescription
        
        manager.fetchAllPostsForCommunity { [weak self] in
            self?.tableView.reloadData()
        }
    }
    
    @objc func onFilterButtonTap() {
        let alertController = UIAlertController(title: nil, message: nil, preferredStyle: .actionSheet)
        let option1 = UIAlertAction(title: "Published | First Created", style: .default) { [weak self] _ in
            
            self?.fetchCommunityFeed(feedType: .published, feedSortOption: .firstCreated)
        }
        
        let option2 = UIAlertAction(title: "Published | Last Created", style: .default) { [weak self] _ in
            
            self?.fetchCommunityFeed(feedType: .published, feedSortOption: .lastCreated)
        }
        
        let option3 = UIAlertAction(title: "Reviewing | First Created", style: .default) { [weak self] _ in
            
            self?.fetchCommunityFeed(feedType: .reviewing, feedSortOption: .firstCreated)
        }
        
        let option4 = UIAlertAction(title: "Reviewing | Last Created", style: .default) { [weak self] _ in
            
            self?.fetchCommunityFeed(feedType: .reviewing, feedSortOption: .lastCreated)
        }
        
        let option5 = UIAlertAction(title: "Declined | Last Created", style: .default) { [weak self] _ in
            
            self?.fetchCommunityFeed(feedType: .declined, feedSortOption: .lastCreated)
        }
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel)
        
        alertController.addAction(option1)
        alertController.addAction(option2)
        alertController.addAction(option3)
        alertController.addAction(option4)
        alertController.addAction(option5)
        
        alertController.addAction(cancelAction)
        present(alertController, animated: true, completion: nil)
    }
    
    func showAllCommunityEvents(isSubscribeAction: Bool) {
        var actions = [UIAlertAction]()
        
        let events: [AmityCommunityEvent] = [.community, .posts , .postsAndComments, .comments, .storyAndComments]
        for event in events {
            let action = UIAlertAction(title: getEventDescription(event: event), style: .default) { [weak self] action in
                
                guard let strongSelf = self else { return }
                
                if isSubscribeAction {
                    self?.manager.subscribeEvent(event: event, completion: { isSuccess in
                        
                        let message = isSuccess ? "Subscribed Successfully" : "Failed to subscribe event"
                        AppUtility.showAlert(in: strongSelf, title: "", message: message, action: nil)
                    })
                } else {
                    
                    self?.manager.unsubscribeEvent(event: event, completion: { isSuccess in
                        let message = isSuccess ? "Unsubscribed Successfully" : "Failed to unsubscribe event"
                        AppUtility.showAlert(in: strongSelf, title: "", message: message, action: nil)
                    })
                }
            }
            actions.append(action)
        }
        
        AppUtility.showActionSheet(in: self, title: "Events", message: "Subscribe to events", actions: actions)
    }
    
    func getEventDescription(event: AmityCommunityEvent) -> String {
        switch event {
        case .community:
            return "Community"
        case .posts:
            return "Posts"
        case .comments:
            return "Comments"
        case .postsAndComments:
            return "Post And Comments"
        case .storyAndComments:
            return "Story And Comments"
        default:
            return ""
        }
    }
}

extension CommunityRealTimeEventController: UITableViewDataSource, UITableViewDelegate {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return manager.elements.count
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        let element = manager.elements[section]
        return manager.getRowCount(element: element)
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let sectionElement = manager.elements[indexPath.section]
        
        switch sectionElement {
        case .header:
            let cell = tableView.dequeueReusableCell(withIdentifier: RealTimeEventHeaderCell.identifier) as! RealTimeEventHeaderCell
            
            cell.modelDetailLabel.numberOfLines = 0
            cell.modelDetailLabel.text = manager.community.modelDescription
            
            cell.subscribeAction = { [weak self] in
                self?.showAllCommunityEvents(isSubscribeAction: true)
            }
            cell.unsubscribeAction = { [weak self] in
                self?.showAllCommunityEvents(isSubscribeAction: false)
            }
            
            return cell
        case .posts:
            let cell = tableView.dequeueReusableCell(withIdentifier: UITableViewCell.identifier)!
            let post = manager.posts[indexPath.row]
            
            cell.textLabel?.numberOfLines = 0
            cell.textLabel?.text = post.modelDescription
            
            return cell
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableView.automaticDimension
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let sectionElement = manager.elements[indexPath.section]
        guard sectionElement != .header else { return }
        
        let postRealTimeController = PostRealTimeEventController()
        let manager = PostRealTimeEventManager(post: manager.posts[indexPath.row], isObserveMode: false)
        postRealTimeController.manager = manager
        
        self.navigationController?.pushViewController(postRealTimeController, animated: true)
    }
}
