//
//  LatestCommentView.swift
//  SampleApp
//
//  Created by Nishan Niraula on 3/4/21.
//  Copyright © 2021 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct LatestCommentView: View {
    @State var referenceId: String = ""
    @State var selectedReferenceType: String = AmityCommentReferenceType.post.identifier
    @State var shouldIncludeReplies: Bool = false
    
    @ObservedObject var viewModel = LatestCommentViewModel()
    
    var referenceTypes: [String] = [AmityCommentReferenceType.post.identifier, AmityCommentReferenceType.content.identifier]
    
    var body: some View {
        Form {
            Section(header: Text("Create Comment")) {
                TextField("Enter Post/Content Id", text: $referenceId)
                Toggle("Include Replies", isOn: $shouldIncludeReplies)
                
                Picker("Reference Type", selection: $selectedReferenceType) {
                    ForEach(referenceTypes, id: \.self) { item in
                        Text(item)
                    }
                }
            }
            
            Section {
                Button(action: {
                    viewModel.getLatestComment(referenceId: referenceId, referenceType: selectedReferenceType, includeReplies: shouldIncludeReplies)
                }, label: {
                    HStack {
                        Spacer()
                        Text("Get Latest Comment")
                        Spacer()
                    }
                })
            }
            
            Section(header: Text("Here you will see local & server result received from sdk")) {
                VStack(alignment: .leading) {
                    Text("Local Result")
                        .font(.system(size: 18))
                        .fontWeight(.bold)
                        .padding(.bottom, 12)
                    Text(viewModel.localOutput)
                }
                
                VStack(alignment: .leading) {
                    Text("Server Result")
                        .font(.system(size: 18))
                        .fontWeight(.bold)
                        .padding(.bottom, 12)
                    Text(viewModel.serverOutput)
                }
            }
        }
        .navigationBarTitle("Latest Comment", displayMode: .inline)
    }
}

struct LatestCommentView_Previews: PreviewProvider {
    static var previews: some View {
        LatestCommentView()
    }
}

class LatestCommentViewModel: ObservableObject {
    
    var commentRepo: AmityCommentRepository = AmityCommentRepository(client: AmityManager.shared.client!)
    
    @Published var localOutput = ""
    @Published var serverOutput = ""
    
    func getLatestComment(referenceId: String, referenceType: String, includeReplies: Bool) {
        let actualType: AmityCommentReferenceType = AmityCommentReferenceType.post.identifier == referenceType ? .post : .content
        Task { @MainActor in
            do {
                let comment = try await commentRepo.getLatestComment(withReferenceId: referenceId, referenceType: actualType, includeReplies: includeReplies)
                
                let commentData = comment.data
                let commentId = comment.commentId
                let commentUserId = comment.userId
                
                let dataToPrint = """
                 Error: nil

                 ...

                 Comment Data: \(commentData ?? [:])
                 Comment Id: \(commentId)
                 Comment User Id: \(commentUserId)
                 """
    
                self.serverOutput = dataToPrint
            } catch {
                let dataToPrint = """
                        Error: \(String(describing: error))
                        
                        Result: There is no latest comment.
                        """
                
                
        
                self.serverOutput = dataToPrint
            }
        }
    }
}
