//
//  MoreTableViewController.swift
//  SampleApp
//
//  Created by Federico Zanetello on 6/6/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import Foundation
import SwiftUI
import AmitySDK

final class MoreTableViewController: UITableViewController {
    /// To be injected.
    weak var client: AmityClient!

    lazy var pushNotificationManager = PushNotificationRegistrationManager(client: self.client)
    
    private var channelRepository: AmityChannelRepository?
    
    private var createToken: AmityNotificationToken?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        super.prepare(for: segue, sender: sender)
        switch segue.destination {
        case let userPushNotificationsViewController as UserLevelPushNotificationsTableViewController:
            userPushNotificationsViewController.client = client
        case let notificationRegistrationTableViewController as NotificationRegistrationTableViewController:
            notificationRegistrationTableViewController.client = client
        default:
            break
        }
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        
        tableView.deselectRow(at: indexPath, animated: true)
        
        switch (indexPath.section,indexPath.row) {
        case (1, 1):
            let contentSettingsViewController = UIHostingController(rootView: ContentSettingsView(client: self.client))
            navigationController?.pushViewController(contentSettingsViewController, animated: true)
        case (2,0):
            // Update metadata
            updateUserMeta()
            
        case (2, 1):
            // Unregister
            pushNotificationManager.unregisterUser { _ in
            }
            
            pushNotificationManager.unregisterDevice { _ in
            }
            
            // Disconnect
            Task { @MainActor in
                do {
                    try await client.secureLogout()
                    
                    showAlert(title: "Logout Successful", message: "Session is now disconnected. Everything related to this user should be cleared.")
                } catch {
                    showAlert(title: "Logout Error", message: error.localizedDescription)
                    
                }
            }
            
        case (2,2):
            showInputAlert(title: "Login New User", message: "This will login new user without calling unregister for previous user", placeholder: "Enter User Id:") { userId in
                Task { @MainActor in
                    do {
                        let authToken: String?
                        if AmityManager.shared.needsAuthToken {
                            authToken = try await AmityManager.shared.getAuthToken(userId: userId)
                        } else {
                            authToken = nil
                        }
                        try await self.client.login(userId: userId, displayName: nil, authToken: authToken, sessionHandler: MySessionHandler())
                        self.showAlert(title: "Login Response", message: "Successful")
                    } catch {
                        self.showAlert(title: "Login Response", message: "Failed \(error.localizedDescription)")
                    }
                }
            }
        case (2,3):
            client.disconnect()
            showAlert(title: "Disconnected", message: "SDK disconnects from the server without logging out the user.")
            
        case (3, 0):
            let createUserTokenViewController = CreateUserTokenViewController()
            let navigationController = UINavigationController(rootViewController: createUserTokenViewController)
            navigationController.modalPresentationStyle = .fullScreen
            present(navigationController, animated: true, completion: nil)
            
        default:
            break
        }
        
    }
    
    private func updateUserMeta() {
        showInputAlert(title: "Update user meta", message: "", placeholder: "Type meta here") { meta in
            
            let data = [
                "meta": meta
            ]
            UserUpdateManager.shared.updateMetadata(metadata: data, completion: nil)
        }
    }
    
    private func showAlert(title: String, message: String) {
        let alertController = UIAlertController(title: title, message: message, preferredStyle: .alert)
        alertController.addAction(UIAlertAction(title: "Dismiss", style: .cancel, handler: nil))
        
        present(alertController, animated: true, completion: nil)
    }
    
    private func showInputAlert(title: String, message: String, placeholder: String, completion: @escaping (_ input: String) -> Void) {
        let alertController = UIAlertController(title: title, message: message, preferredStyle: .alert)
        alertController.addTextField { textField in
            textField.placeholder = placeholder
        }
        
        let addAction = UIAlertAction(title: "Submit", style: .default) { action in
            guard let meta = alertController.textFields?.first?.text, !meta.isEmpty else { return }
            completion(meta)
        }
        
        alertController.addAction(addAction)
        alertController.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: nil))
        
        present(alertController, animated: true, completion: nil)
    }
}
